const request = require('request');

class RasaAccessor {
    constructor(RASA_URL, RASA_CONFIDENCE_THRESHOLD){
        this.RASA_URL = RASA_URL;
        this.RASA_CONFIDENCE_THRESHOLD = RASA_CONFIDENCE_THRESHOLD;
    };

    resolveText(data) {

        return new Promise(async resolve => {
            let {text, companyId} = data;
            companyId = this.parseCompanyId(companyId);
            const url = this.RASA_URL + '/parse';
            try {
                const result = await this.requestData(url, {q: text, project: companyId});
                const response = JSON.parse(result.body);
                if (response.intent.confidence >= this.RASA_CONFIDENCE_THRESHOLD &&
                    this.hasEntityConfident(response.entities, this.RASA_CONFIDENCE_THRESHOLD)) {

                    let intentBot = data.intents.find(item => {
                        return item.intent == response.intent.name;
                    });
                    if (intentBot) {
                        resolve(intentBot);
                    }
                }
            } catch (e) {
                // console.log('Erro no Rasa ', e);
            }
            return resolve(data.findIntent);
        })

    }

    parseCompanyId(name) {
        let companyId = name;
        if (companyId.indexOf('.') > -1) {
            const splited = companyId.split('.');
            companyId = splited[splited.length - 1];
        }
        return companyId;
    }

    requestData(url, qs) {
        return new Promise(((resolve, reject) => {
            request.get(url, {qs}, (err, res) => {
                if (err) {reject(err)}
                resolve(res);
            });
        }));
    }

    hasEntityConfident(entities, threshold) {
        for (const entity of entities) {
            if (entity.value === entity.entity || (entity.confidence && entity.confidence >= threshold && entity.processors &&
                entity.processors.indexOf('ner_synonyms') > -1)) { // entry is entity or synonym

                return true;
            }
        }
        return false;
    }
}

module.exports = RasaAccessor;
