
const initializeTranslateObject = require('./translate-object');
const Translate = require('@google-cloud/translate');
const _ = require('lodash')

const defaultAttributes = ['chatTreeId', 'question', 'dateEnd', 'dateInit', 'isLink', 'holiday', 'trackId', 'dialog', 'intent', 'tags', 'goToDialog', 'type', 'search', 'image', 'registerData', 'id', 'subject', 'to', 'default', 'hsystemKey', 'statusRevision', 'dialogNewRequest', 'compareSuites', 'url', 'otherParams', 'cupom', 'typeBookingEngine', 'randomId'];

module.exports = function (config) {

    let translator = {};

    const translateObject = initializeTranslateObject({
        url: config.INFOCHAT_DATA_SERVER + '/',
        redis: {
            url: config.REDIS_URL,
            db: 0 // translates
        }
    });

    translator.translate = function (data, languageTo, languageFrom) {
        const chatTreeId = data.chatTreeId;
        const json = data.json;

        return translateObject(json, languageFrom, languageTo, defaultAttributes, chatTreeId)
            .then(json =>
                Object.assign({}, data, { json })
            )
            .catch(error => {
                console.log('error', error);
                return data;
            });
    };



    translator.googleTranslateIfNeed = function(text, target){

        const translate = new Translate({
            key: config.GOOGLE_TRANSLATE_KEY
        });

        return new Promise(function(resolve, reject){

            translate
                .translate(text, {from: target, to: "pt-br"})
                .then(results => {
                    const translation = results[0];

                    console.log(`Text: ${text}`);
                    console.log(`Translation: ${translation}`);

                    resolve(translation);

                })
                .catch(err => {
                    console.error('ERROR:', err);
                    reject(err);
                });

        })

    };

    return translator;
};
