Promise = require('bluebird');
const _ = require('lodash');

const asyncMapDeep = (array, key, mapFn) =>
    Promise.map(
        array,
        value => (_.isObject(value) || _.isArray(value))
            ? asyncMapValuesDeep(value, mapFn)
            : mapFn(value, key),
        { concurrency: 100 }
    );

const asyncMapValuesDeep = (object, mapFn) =>
    Promise.props(
        _.mapValues(object, (value, key) =>
            _.isArray(value)
                ? asyncMapDeep(value, key, mapFn)
                : _.isObject(value)
                    ? asyncMapValuesDeep(value, mapFn)
                    : mapFn(value, key)
        )
    );

module.exports = {
    asyncMapValuesDeep
};
